Turninator = LibStub("AceAddon-3.0"):NewAddon("Turninator", "AceConsole-3.0", "AceEvent-3.0")

BINDING_HEADER_TURNINATOR = "Turninator"
BINDING_NAME_TURNINATORGO = "GO"

local DL = {
	[0] = "none",
	"error",
	"test",
	"config",
	"spam",
}

local dbgprint = { squelch = function(...) return nil end }
local function SetDebugLevel(iDL)
	for i = 0,     iDL do dbgprint[ DL[i] ] = print; end
	for i = iDL+1, #DL do dbgprint[ DL[i] ] = dbgprint.squelch; end
end


local dlgAction = nil


-- Called when the addon is loaded
function Turninator:OnInitialize()
	self.db = Turninator.LoadSavedSettings()
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Turninator", Turninator.GetUIOptions, { "Turninator", "tr" } )

	for k,v in pairs(DL) do DL[v] = k end
	SetDebugLevel(self.db.global.iDebugLevel)
end


-- Called when the addon is enabled
function Turninator:OnEnable()
	if (self.db.profile.fEnabled) then
		dbgprint.spam("enabling addon")
		self:RegisterEvent("GOSSIP_SHOW",     "DialogEventHandler")
		self:RegisterEvent("GOSSIP_CLOSED",   "DialogEventHandler")
		self:RegisterEvent("QUEST_GREETING",  "DialogEventHandler")
		self:RegisterEvent("QUEST_DETAIL",    "DialogEventHandler")
		self:RegisterEvent("QUEST_PROGRESS",  "DialogEventHandler")
		self:RegisterEvent("QUEST_COMPLETE",  "DialogEventHandler")
		self:RegisterEvent("QUEST_FINISHED",  "DialogEventHandler")

		self:RegisterEvent("PLAYER_TARGET_CHANGED", "SelectBinding")
		self:SelectBinding("OnEnable")
	end
end

-- Called when the addon is Disabled
function Turninator:OnDisable()
	if (self.db.profile.fEnabled) then
		dbgprint.spam("disabling addon")
		dlgAction = nil;
		
		self:UnregisterEvent("GOSSIP_SHOW")
		self:UnregisterEvent("GOSSIP_CLOSED")
		self:UnregisterEvent("QUEST_GREETING")
		self:UnregisterEvent("QUEST_DETAIL")
		self:UnregisterEvent("QUEST_PROGRESS")
		self:UnregisterEvent("QUEST_COMPLETE")
		self:UnregisterEvent("QUEST_FINISHED")

		self:UnregisterEvent("PLAYER_TARGET_CHANGED")
		self:SelectBinding("OnDisable")
	end
end

function Turninator:Config()
	dbgprint.error("debug reporting level set to", DL[self.db.global.iDebugLevel])
	self:Print("addon is", self.db.profile.fEnabled and "enabled" or "disabled")
	self:Print("shared cmd is", self.db.profile.strSharedCmd or "not set")
	self:Print("shared key is", self.db.profile.strSharedKey or "not set")
	if (self.db.profile.strSharedKey) then 
		dbgprint.spam("shared key currently bound to", GetBindingByKey(self.db.profile.strSharedKey))
	end
end


function Turninator:Debug(info, value)
	local strValue, index = self:GetArgs(value, 1);
	
	if (strValue ~= nil) then
		if (index ~= 1e9) then
			print("ERROR: too many parameters passed");
		end

		local iValue = tonumber(strValue)
		if (iValue == nil) then
			print("ERROR: no value given for debug level");
			return;
		elseif (DL[iValue] == nil) then
			print("ERROR: level_value must be number between 0 and", #DL);
			return;
		else
			self.db.global.iDebugLevel = iValue
			SetDebugLevel(iValue)
		end
	end
	
	self:Config()
end

function Turninator:ShareKey(info, value)
	local strKey, index = self:GetArgs(value, 1);
	
	if (strKey == nil) then
		print("ERROR: no key specified");
		return;
	elseif (index ~= 1e9) then
		print("ERROR: too many parameters passed");
		return
	end

	local strAction = GetBindingByKey(strKey)
	if (not SetBinding(strKey, strAction)) then
		print("ERROR: invalid key")
		return;
	end
	
	self:Reset(true);  -- release old binding before setting up new one
	
	if (not strAction or strAction == "") then
		print("sharing previously unbound key");
	else
		print("sharing and sharing key", strKey, "with action", strAction);
		self.db.profile.strSharedCmd = strAction
	end
	self.db.profile.strSharedKey = strKey
	
	self:SelectBinding("ShareKey")
	self:Config()
end
	
function Turninator:ShareCmd(info, value)
	local strAction, index = self:GetArgs(value, 1);
	
	if (strAction == nil) then
		print("ERROR: no action specified");
		return;
	elseif (index ~= 1e9) then
		print("ERROR: too many parameters passed");
		return
	end

	local strKey = GetBindingKey(strAction)
	if (not strKey or strKey == "") then
		print("action does not exist or is unbound");
		return;
	end

	self:Reset(true);  -- release old binding before setting up new one
	
	print("sharing and sharing key", strKey, "with action", strAction);
	self.db.profile.strSharedKey = strKey
	self.db.profile.strSharedCmd = strAction
	
	self:SelectBinding("ShareCmd")
	self:Config()
end

function Turninator:Reset(silent)
	if (self.db.profile.strSharedKey) then
		SetBinding(self.db.profile.strSharedKey, nil);
		SetBinding(self.db.profile.strSharedKey, self.db.profile.strSharedCmd);
	end

	self.db.profile.strSharedKey = nil
	self.db.profile.strSharedCmd = nil
	
	if not silent then self:Config() end
end



function Turninator:go()
	if not dlgAction then 
		print("foo");
		return;
	end

	if (type(dlgAction)	~= "table") then
		dbgprint.error("dlgAction is not table. its a", type(dlgAction));
	elseif (not dlgAction[1] or type(dlgAction[1]) ~= "function") then
		dbgprint.error("dlgAction does not contain function");
	else
		dlgAction[1](unpack(dlgAction, 2));
	end
end


function Turninator:SelectBinding(event)
	if (self.db.profile.strSharedKey == nil) then 
		dbgprint.spam("no key is set; cannot respond to", event);
	elseif (self.db.profile.fEnabled and dlgAction ~= nil) then
		dbgprint.spam("taking binding for dialog event", event);
		SetBinding(self.db.profile.strSharedKey, "TURNINATORGO");
	elseif (self.db.profile.fEnabled and self:CanInteract()) then
		dbgprint.spam(event, "taking binding for interaction");
		SetBinding(self.db.profile.strSharedKey, "INTERACTTARGET")
	else
		dbgprint.spam(event, "releasing binding");
		SetBinding(self.db.profile.strSharedKey, self.db.profile.strSharedCmd or "INTERACTTARGET")
	end
end


function GetDialogAction(event, ...)

	if (event == "GOSSIP_SHOW") then
		local function GetActive(...)
			local tbl = {}
			for iOffset = 1, select("#", ...), 4 do
				local iActive = ceil(iOffset/4)
				local strQuest = select(iOffset, ...)
				local fComplete = select(iOffset+3, ...) ~= nil
				tbl[iActive] = { name = strQuest, complete = fComplete }
			end
			return tbl;
		end

		local function GetGossip(...)
			local tbl = {}
			for iOffset = 1, select("#", ...), 2 do
				local strType = select(iOffset+1, ...)
				if (strType == "gossip") then
					tbl[#tbl+1] = ceil(iOffset/2)
				end
			end
			return tbl;
		end

		local tblActive = GetActive(GetGossipActiveQuests())
		for i = 1, #tblActive do
			if (tblActive[i].complete) then
				-- found a quest we can turn in, that's what button will do
				dbgprint.spam("will turn in quest", i, tblActive[i].name);
				return { SelectGossipActiveQuest, i }
			end
		end
		
		-- no quests we can turn in, so how about pick up an available quest
		local cAvailable = GetNumGossipAvailableQuests()
		dbgprint.spam(cAvailable, "available quests for this npc")
		if (cAvailable > 0) then
			return { SelectGossipAvailableQuest, 1 }
		end

		-- if the npc has chatty gossip options try the first one
		if (GetNumGossipOptions() > 0) then
			local tblGossip = GetGossip(GetGossipOptions())
			-- PrintTable(tblGossip)
			dbgprint.spam(#tblGossip, "gossip text options for this npc")
			if (tblGossip[1] ~= nil) then 
				return { SelectGossipOption, tblGossip[1] }
			end
		end
		
		-- nothing we can do with this quest giver apparently ... exit seems iffy ... revisit
		return { CloseGossip }

	elseif (event == "QUEST_GREETING") then
		local function GetActive()
			local tbl = {}
			for iActive = 1, GetNumActiveQuests() do
				local strQuest, fComplete = GetActiveTitle(iActive)
				tbl[iActive] = { name = strQuest, complete = (fComplete ~= nil) }
			end
			return tbl;
		end

		local tblActive = GetActive()
		for i = 1, #tblActive do
			if (tblActive[i].complete) then
				-- found a quest we can turn in, that's what button will do
				dbgprint.spam("will turn in quest", i, tblActive[i].name);
				return { SelectActiveQuest, i }
			end
		end
		
		-- no quests we can turn in, so how about pick up an available quest
		local cAvailable = GetNumAvailableQuests()
		dbgprint.spam(cAvailable, "available quests for this npc")
		if (cAvailable > 0) then
			return { SelectAvailableQuest, 1 }
		end
		
		-- nothing we can do with this quest giver apparently ... exit seems iffy ... revisit
		return { CloseQuest }

	elseif (event == "QUEST_DETAIL") then
		return { AcceptQuest }

	elseif (event == "QUEST_PROGRESS") then
		if (IsQuestCompletable()) then
			return { CompleteQuest }
		else
			return { DeclineQuest }
		end
		-- todo - can't complete, back up?

	elseif (event == "QUEST_COMPLETE") then
		local function CheckRewardSelection()
			if (QuestInfoFrame.itemChoice ~= 0) then
				return GetQuestReward(QuestInfoFrame.itemChoice)
			else
				return QuestChooseRewardError()
			end 
		end
			
		if (GetNumQuestChoices() == 0) then
			return { GetQuestReward }
		else
			return { CheckRewardSelection }
		end

	elseif (event == "GOSSIP_CLOSED") then
		return nil

	elseif (event == "QUEST_FINISHED") then
		return nil
	end
	
	return nil
end

function Turninator:DialogEventHandler(event, ...)
	if (self.db.profile.strSharedKey == nil) then 
		return; 
	end
	
	dlgAction = GetDialogAction(event, ...)
	self:SelectBinding(event);
end


function Turninator:CanInteract()
	return (UnitExists("target") and UnitIsFriend("player", "target") and 
	    not UnitIsCorpse("target") and not UnitIsDead("target") and
	    not UnitPlayerControlled("target") and not UnitCanAttack("player", "target"));
end

	    

